#include <cxxtest/TestSuite.h>

#include "Engine.h"
#include "MockSelectorFactory.h"
#include "MockServerSocketFactory.h"
#include "MockTruth.h"
#include "MockWeaverFactory.h"
#include "ThreadStateGraph.h"

class MockForEngine :
	public MockServerSocketFactory,
	public MockSelectorFactory,
	public MockWeaverFactory,
	public MockTruth
{
public:
};

class EngineTestSuite : public CxxTest::TestSuite
{
public:
	MockForEngine *mock;
	MockServerSocket *serverSocket;
	MockSelector *selector;
	MockWeaver *weaver;

	void setUp()
	{
		TS_ASSERT( mock = new MockForEngine );

		serverSocket = &( mock->mockServerSocket );
		selector = &( mock->mockSelector );
		weaver = &( mock->mockWeaver );
	}

	void tearDown()
	{
		TS_ASSERT_THROWS_NOTHING( delete mock );
	}

	void test_constructor()
	{
		Engine *engine;

		TS_ASSERT( engine = new Engine );

		TS_ASSERT( selector->wasCreated );

		TS_ASSERT( weaver->wasCreated );
        TS_ASSERT_EQUALS( weaver->lastSelector, selector );

		TS_ASSERT( serverSocket->wasCreated );
		TS_ASSERT_EQUALS( serverSocket->lastSelector, selector );
		TS_ASSERT_EQUALS( serverSocket->lastPort, EngineConfiguration::BTHREAD_MANAGEMENT_PORT );
		TS_ASSERT_EQUALS( serverSocket->lastAccepter, weaver );

		TS_ASSERT_THROWS_NOTHING( delete engine );
		TS_ASSERT( selector->wasDiscarded );
		TS_ASSERT( serverSocket->wasDiscarded );
		TS_ASSERT( weaver->wasDiscarded );
	}

	void test_entry_point()
	{
		mock->numberOfTruths = 5;

		TS_ASSERT_THROWS_NOTHING( Engine::main() );

		TS_ASSERT_EQUALS( selector->numberOfSelectCalls, 5U );
	}

    void test_add_blocking_data()
    {
        Engine engine;

        Vector<Event> events;
        events.append( Event( 5 ) );
        events.append( Event( 6 ) );

        TS_ASSERT_THROWS_NOTHING( engine.addThreadBlockingData( "Jefferson", events ) );

        TS_ASSERT_EQUALS( weaver->lastThreadName, "Jefferson" );
        TS_ASSERT_EQUALS( weaver->lastBlockableEvents.size(), 2U );
        TS_ASSERT_EQUALS( weaver->lastBlockableEvents[0].type(), 5U );
        TS_ASSERT_EQUALS( weaver->lastBlockableEvents[1].type(), 6U);
    }

    void test_add_blocking_data__state_graph()
    {
        Engine engine;

        ThreadStateGraph threadStateGraph;

        threadStateGraph._events.append( Event( 1 ) );
        threadStateGraph._events.append( Event( 2 ) );

        threadStateGraph._initialState = "a";

        ThreadState a;
        ThreadState b;

        a._label = "a";
        a._requestedEvents.append( Event( 1 ) );
        a._blockedEvents.append( Event( 2 ) );
        a._successors.append( Transition( Event( 1 ), "b" ) );

        b._label = "b";
        b._watchedEvents.append( Event( 2 ) );
        b._successors.append( Transition( Event( 2 ), "b" ) );
        b._predecessors.append( Transition( Event( 1 ), "a" ) );
        b._history.append( Event( 1 ) );

        threadStateGraph._graph["a"] = a;
        threadStateGraph._graph["b"] = b;

        TS_ASSERT_THROWS_NOTHING( engine.addThreadStateGraph( "Jefferson", threadStateGraph ) );

        TS_ASSERT_EQUALS( weaver->lastThreadName, "Jefferson" );
        TS_ASSERT_EQUALS( weaver->lastThreadStateGraph, &threadStateGraph );
    }
};

//
// Local Variables:
// compile-command: "make -C ../.. "
// tags-file-name: "../../TAGS"
// c-basic-offset: 4
// End:
//
